@@ -6,27 +6,31 @@ struct Node<T> {
     next: Option<Rc<Node<T>>>,
 }
 
+/// A persistent stack.
+///
+/// This is similar to `Stack` from the rpds crate, but it's faster
+/// and uses less memory.
 #[derive(Debug, Clone, Default, PartialEq, Eq)]
-pub struct Stack<T> {
+pub(crate) struct Stack<T> {
     head: Option<Rc<Node<T>>>,
 }
 
 impl<T> Stack<T> {
-    pub fn new() -> Self {
+    pub(crate) fn new() -> Self {
         Self { head: None }
     }
 
-    pub fn peek(&self) -> Option<&T> {
+    pub(crate) fn peek(&self) -> Option<&T> {
         self.head.as_deref().map(|n| &n.val)
     }
 
-    pub fn pop(&self) -> Option<Stack<T>> {
+    pub(crate) fn pop(&self) -> Option<Stack<T>> {
         self.head.as_deref().map(|n| Self {
             head: n.next.clone(),
         })
     }
 
-    pub fn push(&self, v: T) -> Stack<T> {
+    pub(crate) fn push(&self, v: T) -> Stack<T> {
         Self {
             head: Some(Rc::new(Node {
                 val: v,
@@ -36,7 +40,7 @@ impl<T> Stack<T> {
     }
 
     // O(n)
-    pub fn size(&self) -> usize {
+    pub(crate) fn size(&self) -> usize {
         let mut res = 0;
         let mut node = &self.head;
         while let Some(next) = node {
@@ -46,7 +50,7 @@ impl<T> Stack<T> {
         res
     }
 
-    pub fn is_empty(&self) -> bool {
+    pub(crate) fn is_empty(&self) -> bool {
         self.head.is_none()
     }
 }
