@@ -15,7 +15,7 @@ use crate::{
         self, apply_colors, apply_line_number_color, color_positions, novel_style, split_and_apply,
         BackgroundColor,
     },
-    lines::{codepoint_len, format_line_num, split_on_newlines, LineNumber},
+    lines::{codepoint_len, format_line_num, LineNumber},
     options::{DisplayMode, DisplayOptions},
     parse::syntax::{zip_pad_shorter, MatchedPos},
     positions::SingleLineSpan,
@@ -65,8 +65,8 @@ fn format_missing_line_num(
 
 /// Display `src` in a single column (e.g. a file removal or addition).
 fn display_single_column(
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: &Option<String>,
     file_format: &FileFormat,
     src_lines: &[String],
     side: Side,
@@ -78,8 +78,8 @@ fn display_single_column(
 
     let mut header_line = String::new();
     header_line.push_str(&style::header(
-        lhs_display_path,
-        rhs_display_path,
+        display_path,
+        old_path,
         1,
         1,
         file_format,
@@ -316,8 +316,8 @@ fn highlight_as_novel(
 pub fn print(
     hunks: &[Hunk],
     display_options: &DisplayOptions,
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: &Option<String>,
     file_format: &FileFormat,
     lhs_src: &str,
     rhs_src: &str,
@@ -345,21 +345,15 @@ pub fn print(
         )
     } else {
         (
-            split_on_newlines(lhs_src)
-                .iter()
-                .map(|s| format!("{}\n", s))
-                .collect(),
-            split_on_newlines(rhs_src)
-                .iter()
-                .map(|s| format!("{}\n", s))
-                .collect(),
+            lhs_src.lines().map(|s| format!("{}\n", s)).collect(),
+            rhs_src.lines().map(|s| format!("{}\n", s)).collect(),
         )
     };
 
     if lhs_src.is_empty() {
         for line in display_single_column(
-            lhs_display_path,
-            rhs_display_path,
+            display_path,
+            old_path,
             file_format,
             &rhs_colored_lines,
             Side::Right,
@@ -372,8 +366,8 @@ pub fn print(
     }
     if rhs_src.is_empty() {
         for line in display_single_column(
-            lhs_display_path,
-            rhs_display_path,
+            display_path,
+            old_path,
             file_format,
             &lhs_colored_lines,
             Side::Left,
@@ -403,8 +397,8 @@ pub fn print(
     let mut prev_lhs_line_num = None;
     let mut prev_rhs_line_num = None;
 
-    let lhs_lines = split_on_newlines(lhs_src);
-    let rhs_lines = split_on_newlines(rhs_src);
+    let lhs_lines = lhs_src.lines().collect::<Vec<_>>();
+    let rhs_lines = rhs_src.lines().collect::<Vec<_>>();
     let matched_lines = all_matched_lines_filled(lhs_mps, rhs_mps, &lhs_lines, &rhs_lines);
     let mut matched_lines_to_print = &matched_lines[..];
 
@@ -412,8 +406,8 @@ pub fn print(
         println!(
             "{}",
             style::header(
-                lhs_display_path,
-                rhs_display_path,
+                display_path,
+                old_path,
                 i + 1,
                 hunks.len(),
                 file_format,
@@ -611,8 +605,10 @@ mod tests {
         let source_dims = SourceDimensions::new(
             80,
             &line_nums,
-            &split_on_newlines("foo\nbar\n"),
-            &split_on_newlines("x\nx\nx\nx\nx\nx\nx\nx\nx\nx\nx\n"),
+            &"foo\nbar\n".lines().collect::<Vec<_>>(),
+            &"x\nx\nx\nx\nx\nx\nx\nx\nx\nx\nx\n"
+                .lines()
+                .collect::<Vec<_>>(),
         );
 
         assert_eq!(source_dims.lhs_line_nums_width, 2);
@@ -627,8 +623,8 @@ mod tests {
                 (Some(0.into()), Some(0.into())),
                 (Some(1.into()), Some(1.into())),
             ],
-            &split_on_newlines("foo\nbar\n"),
-            &split_on_newlines("fox\nbax\n"),
+            &"foo\nbar\n".lines().collect::<Vec<_>>(),
+            &"fox\nbax\n".lines().collect::<Vec<_>>(),
         );
 
         assert_eq!(
@@ -649,8 +645,8 @@ mod tests {
                 (Some(0.into()), Some(0.into())),
                 (Some(1.into()), Some(1.into())),
             ],
-            &split_on_newlines("foo\nbar\n"),
-            &split_on_newlines("fox\nbax\n"),
+            &"foo\nbar\n".lines().collect::<Vec<_>>(),
+            &"fox\nbax\n".lines().collect::<Vec<_>>(),
         );
 
         assert_eq!(
@@ -668,7 +664,7 @@ mod tests {
         // Basic smoke test.
         let res_lines = display_single_column(
             "foo.py",
-            "foo.py",
+            &None,
             &FileFormat::SupportedLanguage(Language::Python),
             &["print(123)\n".to_string()],
             Side::Right,
@@ -724,8 +720,8 @@ mod tests {
         print(
             &hunks,
             &DisplayOptions::default(),
-            "foo-old.el",
             "foo-new.el",
+            &None,
             &FileFormat::SupportedLanguage(Language::EmacsLisp),
             "foo",
             "bar",
