@@ -2,7 +2,7 @@
 
 use crate::{
     constants::Side,
-    lines::{byte_len, split_on_newlines, LineNumber},
+    lines::{byte_len, LineNumber},
     options::DisplayOptions,
     parse::syntax::{AtomKind, MatchKind, MatchedPos, TokenKind},
     positions::SingleLineSpan,
@@ -396,7 +396,7 @@ pub fn apply_colors(
     positions: &[MatchedPos],
 ) -> Vec<String> {
     let styles = color_positions(side, background, syntax_highlight, file_format, positions);
-    let lines = split_on_newlines(s);
+    let lines = s.lines().collect::<Vec<_>>();
     style_lines(&lines, &styles)
 }
 
@@ -450,8 +450,8 @@ pub(crate) fn apply_line_number_color(
 }
 
 pub fn header(
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: &Option<String>,
     hunk_num: usize,
     hunk_total: usize,
     file_format: &FileFormat,
@@ -463,34 +463,24 @@ pub fn header(
         format!("{}/{} --- ", hunk_num, hunk_total)
     };
 
-    let rhs_path_pretty = apply_header_color(
-        rhs_display_path,
+    let display_path_pretty = apply_header_color(
+        display_path,
         display_options.use_color,
         display_options.background_color,
         hunk_num,
     );
-    let lhs_path_pretty = apply_header_color(
-        lhs_display_path,
-        display_options.use_color,
-        display_options.background_color,
-        hunk_num,
-    );
-    if hunk_num == 1 && lhs_display_path != rhs_display_path && display_options.in_vcs {
-        let renamed = format!("Renamed {} to {}", lhs_path_pretty, rhs_path_pretty);
-        format!(
-            "{}\n{} --- {}{}",
-            renamed, rhs_path_pretty, divider, file_format
-        )
-    } else {
-        // Prefer showing the RHS path in the header unless it's
-        // /dev/null. Note that git calls the difftool with
-        // `DIFFTOOL /tmp/git-blob-abc/foo.py foo.py` in some cases.
-        let path_pretty = if rhs_display_path == "/dev/null" {
-            lhs_path_pretty
-        } else {
-            rhs_path_pretty
-        };
-        format!("{} --- {}{}", path_pretty, divider, file_format)
+
+    match old_path {
+        Some(old_path) => {
+            let renamed = format!("Renamed {} to {}", old_path, display_path);
+            format!(
+                "{}\n{} --- {}{}",
+                renamed, display_path, divider, file_format
+            )
+        }
+        None => {
+            format!("{} --- {}{}", display_path_pretty, divider, file_format)
+        }
     }
 }
 
