@@ -40,7 +40,7 @@ extern crate log;
 
 use crate::diff::{dijkstra, unchanged};
 use crate::display::hunks::{matched_pos_to_hunks, merge_adjacent};
-use crate::parse::guess_language::{LANG_EXTENSIONS, LANG_FILE_NAMES};
+use crate::parse::guess_language::language_globs;
 use crate::parse::syntax;
 use diff::changes::ChangeMap;
 use diff::dijkstra::ExceededGraphLimit;
@@ -51,7 +51,7 @@ use files::{
 };
 use log::info;
 use mimalloc::MiMalloc;
-use parse::guess_language::{guess, language_name};
+use parse::guess_language::{guess, language_name, Language, LanguageOverride};
 
 /// The global allocator used by difftastic.
 ///
@@ -67,6 +67,7 @@ use rayon::prelude::*;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
 use std::{env, path::Path};
+use strum::IntoEnumIterator;
 use summary::{DiffResult, FileContent, FileFormat};
 use syntax::init_next_prev;
 use typed_arena::Arena;
@@ -99,13 +100,13 @@ fn main() {
     match options::parse_args() {
         Mode::DumpTreeSitter {
             path,
-            language_override,
+            language_overrides,
         } => {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
             let src = String::from_utf8_lossy(&bytes).to_string();
 
-            let language = language_override.or_else(|| guess(path, &src));
+            let language = guess(path, &src, &language_overrides);
             match language {
                 Some(lang) => {
                     let ts_lang = tsp::from_language(lang);
@@ -119,14 +120,14 @@ fn main() {
         }
         Mode::DumpSyntax {
             path,
-            language_override,
             ignore_comments,
+            language_overrides,
         } => {
             let path = Path::new(&path);
             let bytes = read_or_die(path);
             let src = String::from_utf8_lossy(&bytes).to_string();
 
-            let language = language_override.or_else(|| guess(path, &src));
+            let language = guess(path, &src, &language_overrides);
             match language {
                 Some(lang) => {
                     let ts_lang = tsp::from_language(lang);
@@ -140,30 +141,32 @@ fn main() {
                 }
             }
         }
-        Mode::ListLanguages { use_color } => {
-            for (language, extensions) in LANG_EXTENSIONS {
-                let mut name = language_name(*language).to_string();
+        Mode::ListLanguages {
+            use_color,
+            language_overrides,
+        } => {
+            for (glob, lang_override) in language_overrides {
+                let mut name = match lang_override {
+                    LanguageOverride::Language(lang) => language_name(lang),
+                    LanguageOverride::PlainText => "Text",
+                }
+                .to_string();
+                if use_color {
+                    name = name.bold().to_string();
+                }
+                println!("{} (from override)", name);
+                println!(" {}", glob.as_str());
+            }
+
+            for language in Language::iter() {
+                let mut name = language_name(language).to_string();
                 if use_color {
                     name = name.bold().to_string();
                 }
                 println!("{}", name);
 
-                let mut extensions: Vec<&str> = (*extensions).into();
-                extensions.sort_unstable();
-
-                for extension in extensions {
-                    print!(" *.{}", extension);
-                }
-
-                for (file_language, known_file_names) in LANG_FILE_NAMES {
-                    if file_language == language {
-                        let mut known_file_names: Vec<&str> = (*known_file_names).into();
-                        known_file_names.sort_unstable();
-
-                        for known_file_name in known_file_names {
-                            print!(" {}", known_file_name);
-                        }
-                    }
+                for glob in language_globs(language) {
+                    print!(" {}", glob.as_str());
                 }
                 println!();
             }
@@ -172,11 +175,11 @@ fn main() {
             diff_options,
             display_options,
             set_exit_code,
-            language_override,
+            language_overrides,
             lhs_path,
             rhs_path,
-            lhs_display_path,
-            rhs_display_path,
+            display_path,
+            old_path,
         } => {
             if lhs_path == rhs_path {
                 let is_dir = match &lhs_path {
@@ -220,7 +223,7 @@ fn main() {
                         rhs_path,
                         &display_options,
                         &diff_options,
-                        language_override,
+                        &language_overrides,
                     )
                     .try_for_each_with(send, |s, diff_result| s.send(diff_result))
                     .expect("Receiver should be connected");
@@ -231,14 +234,14 @@ fn main() {
                 }
                 _ => {
                     let diff_result = diff_file(
-                        &lhs_display_path,
-                        &rhs_display_path,
+                        &display_path,
+                        old_path,
                         &lhs_path,
                         &rhs_path,
                         &display_options,
                         &diff_options,
                         false,
-                        language_override,
+                        &language_overrides,
                     );
                     print_diff_result(&display_options, &diff_result);
 
@@ -275,41 +278,41 @@ fn format_num_bytes(num_bytes: usize) -> String {
 
 /// Print a diff between two files.
 fn diff_file(
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: Option<String>,
     lhs_path: &FileArgument,
     rhs_path: &FileArgument,
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
     missing_as_empty: bool,
-    language_override: Option<parse::guess_language::Language>,
+    overrides: &[(glob::Pattern, LanguageOverride)],
 ) -> DiffResult {
     let (lhs_bytes, rhs_bytes) = read_files_or_die(lhs_path, rhs_path, missing_as_empty);
     diff_file_content(
-        lhs_display_path,
-        rhs_display_path,
+        display_path,
+        old_path,
         lhs_path,
         rhs_path,
         &lhs_bytes,
         &rhs_bytes,
         display_options,
         diff_options,
-        language_override,
+        overrides,
     )
 }
 
 fn check_only_text(
     file_format: &FileFormat,
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: Option<String>,
     lhs_src: &str,
     rhs_src: &str,
 ) -> DiffResult {
     let has_changes = lhs_src != rhs_src;
 
     DiffResult {
-        lhs_display_path: lhs_display_path.into(),
-        rhs_display_path: rhs_display_path.into(),
+        display_path: display_path.to_string(),
+        old_path,
         file_format: file_format.clone(),
         lhs_src: FileContent::Text(lhs_src.into()),
         rhs_src: FileContent::Text(rhs_src.into()),
@@ -322,21 +325,21 @@ fn check_only_text(
 }
 
 fn diff_file_content(
-    lhs_display_path: &str,
-    rhs_display_path: &str,
+    display_path: &str,
+    old_path: Option<String>,
     _lhs_path: &FileArgument,
     rhs_path: &FileArgument,
     lhs_bytes: &[u8],
     rhs_bytes: &[u8],
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
-    language_override: Option<parse::guess_language::Language>,
+    overrides: &[(glob::Pattern, LanguageOverride)],
 ) -> DiffResult {
     let (lhs_src, rhs_src) = match (guess_content(lhs_bytes), guess_content(rhs_bytes)) {
         (ProbableFileKind::Binary, _) | (_, ProbableFileKind::Binary) => {
             return DiffResult {
-                lhs_display_path: lhs_display_path.into(),
-                rhs_display_path: rhs_display_path.into(),
+                old_path,
+                display_path: display_path.to_owned(),
                 file_format: FileFormat::Binary,
                 lhs_src: FileContent::Binary,
                 rhs_src: FileContent::Binary,
@@ -351,12 +354,12 @@ fn diff_file_content(
     };
 
     let (guess_src, guess_path) = match rhs_path {
-        FileArgument::NamedPath(_) => (&rhs_src, Path::new(&rhs_display_path)),
-        FileArgument::Stdin => (&rhs_src, Path::new(&lhs_display_path)),
-        FileArgument::DevNull => (&lhs_src, Path::new(&lhs_display_path)),
+        FileArgument::NamedPath(path) => (&rhs_src, Path::new(path)),
+        FileArgument::Stdin => (&rhs_src, Path::new(&display_path)),
+        FileArgument::DevNull => (&lhs_src, Path::new(&display_path)),
     };
 
-    let language = language_override.or_else(|| guess(guess_path, guess_src));
+    let language = guess(guess_path, guess_src, overrides);
     let lang_config = language.map(tsp::from_language);
 
     if lhs_bytes == rhs_bytes {
@@ -368,8 +371,8 @@ fn diff_file_content(
         // If the two files are completely identical, return early
         // rather than doing any more work.
         return DiffResult {
-            lhs_display_path: lhs_display_path.into(),
-            rhs_display_path: rhs_display_path.into(),
+            old_path,
+            display_path: display_path.to_string(),
             file_format,
             lhs_src: FileContent::Text("".into()),
             rhs_src: FileContent::Text("".into()),
@@ -385,13 +388,7 @@ fn diff_file_content(
         None => {
             let file_format = FileFormat::PlainText;
             if diff_options.check_only {
-                return check_only_text(
-                    &file_format,
-                    lhs_display_path,
-                    rhs_display_path,
-                    &lhs_src,
-                    &rhs_src,
-                );
+                return check_only_text(&file_format, display_path, old_path, &lhs_src, &rhs_src);
             }
 
             let lhs_positions = line_parser::change_positions(&lhs_src, &rhs_src);
@@ -420,8 +417,8 @@ fn diff_file_content(
 
                                 let has_syntactic_changes = lhs != rhs;
                                 return DiffResult {
-                                    lhs_display_path: lhs_display_path.into(),
-                                    rhs_display_path: rhs_display_path.into(),
+                                    old_path,
+                                    display_path: display_path.to_string(),
                                     file_format,
                                     lhs_src: FileContent::Text(lhs_src),
                                     rhs_src: FileContent::Text(rhs_src),
@@ -512,8 +509,8 @@ fn diff_file_content(
                             if diff_options.check_only {
                                 return check_only_text(
                                     &file_format,
-                                    lhs_display_path,
-                                    rhs_display_path,
+                                    display_path,
+                                    old_path,
                                     &lhs_src,
                                     &rhs_src,
                                 );
@@ -533,8 +530,8 @@ fn diff_file_content(
                     if diff_options.check_only {
                         return check_only_text(
                             &file_format,
-                            lhs_display_path,
-                            rhs_display_path,
+                            display_path,
+                            old_path,
                             &lhs_src,
                             &rhs_src,
                         );
@@ -563,8 +560,8 @@ fn diff_file_content(
     let has_syntactic_changes = !hunks.is_empty();
 
     DiffResult {
-        lhs_display_path: lhs_display_path.into(),
-        rhs_display_path: rhs_display_path.into(),
+        old_path,
+        display_path: display_path.to_string(),
         file_format,
         lhs_src: FileContent::Text(lhs_src),
         rhs_src: FileContent::Text(rhs_src),
@@ -587,10 +584,11 @@ fn diff_directories<'a>(
     rhs_dir: &'a Path,
     display_options: &DisplayOptions,
     diff_options: &DiffOptions,
-    language_override: Option<parse::guess_language::Language>,
+    overrides: &[(glob::Pattern, LanguageOverride)],
 ) -> impl ParallelIterator<Item = DiffResult> + 'a {
     let diff_options = diff_options.clone();
     let display_options = display_options.clone();
+    let overrides: Vec<_> = overrides.into();
 
     // We greedily list all files in the directory, and then diff them
     // in parallel. This is assuming that diffing is slower than
@@ -604,14 +602,14 @@ fn diff_directories<'a>(
         let rhs_path = Path::new(rhs_dir).join(&rel_path);
 
         diff_file(
-            &rel_path.to_string_lossy(),
-            &rel_path.to_string_lossy(),
+            &rel_path.display().to_string(),
+            None,
             &FileArgument::NamedPath(lhs_path),
             &FileArgument::NamedPath(rhs_path),
             &display_options,
             &diff_options,
             true,
-            language_override,
+            &overrides,
         )
     })
 }
@@ -626,8 +624,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                     println!(
                         "{}",
                         display::style::header(
-                            &summary.lhs_display_path,
-                            &summary.rhs_display_path,
+                            &summary.display_path,
+                            &summary.old_path,
                             1,
                             1,
                             &summary.file_format,
@@ -653,8 +651,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 println!(
                     "{}",
                     display::style::header(
-                        &summary.lhs_display_path,
-                        &summary.rhs_display_path,
+                        &summary.display_path,
+                        &summary.old_path,
                         1,
                         1,
                         &summary.file_format,
@@ -682,8 +680,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                         &summary.lhs_positions,
                         &summary.rhs_positions,
                         hunks,
-                        &summary.lhs_display_path,
-                        &summary.rhs_display_path,
+                        &summary.display_path,
+                        &summary.old_path,
                         &summary.file_format,
                     );
                 }
@@ -691,8 +689,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                     display::side_by_side::print(
                         hunks,
                         display_options,
-                        &summary.lhs_display_path,
-                        &summary.rhs_display_path,
+                        &summary.display_path,
+                        &summary.old_path,
                         &summary.file_format,
                         lhs_src,
                         rhs_src,
@@ -707,8 +705,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
                 println!(
                     "{}",
                     display::style::header(
-                        &summary.lhs_display_path,
-                        &summary.rhs_display_path,
+                        &summary.display_path,
+                        &summary.old_path,
                         1,
                         1,
                         &FileFormat::Binary,
@@ -728,8 +726,8 @@ fn print_diff_result(display_options: &DisplayOptions, summary: &DiffResult) {
             println!(
                 "{}",
                 display::style::header(
-                    &summary.lhs_display_path,
-                    &summary.rhs_display_path,
+                    &summary.display_path,
+                    &summary.old_path,
                     1,
                     1,
                     &FileFormat::Binary,
@@ -752,14 +750,14 @@ mod tests {
         let s = "foo";
         let res = diff_file_content(
             "foo.el",
-            "foo.el",
+            None,
             &FileArgument::from_path_argument(OsStr::new("foo.el")),
             &FileArgument::from_path_argument(OsStr::new("foo.el")),
             s.as_bytes(),
             s.as_bytes(),
             &DisplayOptions::default(),
             &DiffOptions::default(),
-            None,
+            &[],
         );
 
         assert_eq!(res.lhs_positions, vec![]);
