@@ -5,18 +5,17 @@
 //! particularly its
 //! [languages.yml](https://github.com/github/linguist/blob/master/lib/linguist/languages.yml).
 //!
-//! Difftastic does not reuse languages.yml directly, for
-//! implementation simplicity and to avoid parsing very large files
-//! (e.g. package.lock) that can't be handled in a reasonable time
-//! yet.
+//! Difftastic does not reuse languages.yml directly. Linguist has a
+//! larger set of language detection strategies.
 
 use lazy_static::lazy_static;
 use regex::Regex;
-use std::{borrow::Borrow, ffi::OsStr, path::Path};
+use std::{borrow::Borrow, path::Path};
+use strum::{EnumIter, IntoEnumIterator};
 
 /// Languages supported by difftastic. Each language here has a
 /// corresponding tree-sitter parser.
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, EnumIter)]
 pub enum Language {
     Ada,
     Bash,
@@ -43,9 +42,9 @@ pub enum Language {
     Janet,
     Java,
     JavaScript,
+    JavascriptJsx,
     Json,
     Julia,
-    Jsx,
     Kotlin,
     Lua,
     Make,
@@ -63,15 +62,41 @@ pub enum Language {
     Ruby,
     Rust,
     Scala,
+    Solidity,
     Sql,
     Swift,
     Toml,
-    Tsx,
     TypeScript,
+    TypeScriptTsx,
     Yaml,
     Zig,
 }
 
+#[derive(Debug, Clone, Copy)]
+pub enum LanguageOverride {
+    Language(Language),
+    PlainText,
+}
+
+/// If there is a language called `name` (comparing case
+/// insensitively), return it. Treat `"text"` as an additional option.
+pub fn language_override_from_name(name: &str) -> Option<LanguageOverride> {
+    let name = name.trim().to_lowercase();
+
+    if name == "text" {
+        return Some(LanguageOverride::PlainText);
+    }
+
+    for language in Language::iter() {
+        let lang_name = language_name(language);
+        if lang_name.to_lowercase() == name {
+            return Some(LanguageOverride::Language(language));
+        }
+    }
+
+    None
+}
+
 /// The language name shown to the user.
 pub fn language_name(language: Language) -> &'static str {
     match language {
@@ -100,9 +125,9 @@ pub fn language_name(language: Language) -> &'static str {
         Janet => "Janet",
         Java => "Java",
         JavaScript => "JavaScript",
+        JavascriptJsx => "JavaScript JSX",
         Json => "JSON",
         Julia => "Julia",
-        Jsx => "JavaScript JSX",
         Kotlin => "Kotlin",
         Lua => "Lua",
         Make => "Make",
@@ -120,11 +145,12 @@ pub fn language_name(language: Language) -> &'static str {
         Ruby => "Ruby",
         Rust => "Rust",
         Scala => "Scala",
+        Solidity => "Solidity",
         Sql => "SQL",
         Swift => "Swift",
         Toml => "TOML",
-        Tsx => "TypeScript TSX",
         TypeScript => "TypeScript",
+        TypeScriptTsx => "TypeScript TSX",
         Yaml => "YAML",
         Zig => "Zig",
     }
@@ -132,104 +158,23 @@ pub fn language_name(language: Language) -> &'static str {
 
 use Language::*;
 
-/// Which file extensions are associated with which languages.
-pub const LANG_EXTENSIONS: &[(Language, &[&str])] = &[
-    (Ada, &["ada", "adb", "ads"]),
-    (
-        Bash,
-        &[
-            "sh", "bash", "bats", "cgi", "command", "env", "fcgi", "ksh", "sh.in", "tmux", "tool",
-            "zsh",
-        ],
-    ),
-    (C, &["c"]),
-    (
-        Clojure,
-        &[
-            "bb", "boot", "clj", "cljc", "clje", "cljs", "cljx", "edn", "joke", "joker",
-        ],
-    ),
-    (CMake, &["cmake", "cmake.in"]),
-    (CommonLisp, &["lisp", "lsp", "asd"]),
-    // Treat .h as C++ rather than C. This is an arbitrary choice, but
-    // C++ is more widely used than C according to
-    // https://madnight.github.io/githut/
-    (CPlusPlus, &["cc", "cpp", "h", "hh", "hpp", "ino", "cxx"]),
-    (CSharp, &["cs"]),
-    (Css, &["css"]),
-    (Dart, &["dart"]),
-    (Elm, &["elm"]),
-    (EmacsLisp, &["el"]),
-    (Elixir, &["ex", "exs"]),
-    (Elvish, &["elv"]),
-    // TODO: confirm that we support multiple extensions for files
-    // like foo.app.src.
-    (
-        Erlang,
-        &["erl", "app.src", "es", "escript", "hrl", "xrl", "yrl"],
-    ),
-    (Gleam, &["gleam"]),
-    (Go, &["go"]),
-    (Hack, &["hack", "hck", "hhi"]),
-    (Hare, &["ha"]),
-    (Haskell, &["hs"]),
-    (Hcl, &["hcl", "nomad", "tf", "tfvars", "workflow"]),
-    (Html, &["html", "htm", "xhtml"]),
-    (Janet, &["janet", "jdn"]),
-    (Java, &["java"]),
-    (JavaScript, &["cjs", "js", "mjs"]),
-    (
-        Json,
-        &[
-            "json",
-            "avsc",
-            "geojson",
-            "gltf",
-            "har",
-            "ice",
-            "JSON-tmLanguage",
-            "jsonl",
-            "mcmeta",
-            "tfstate",
-            "tfstate.backup",
-            "topojson",
-            "webapp",
-            "webmanifest",
-        ],
-    ),
-    (Jsx, &["jsx"]),
-    (Julia, &["jl"]),
-    (Kotlin, &["kt", "ktm", "kts"]),
-    (Lua, &["lua"]),
-    (Make, &["mak", "d", "make", "makefile", "mk", "mkfile"]),
-    (Newick, &["nhx", "nwk", "nh"]),
-    (Nix, &["nix"]),
-    (OCaml, &["ml"]),
-    (OCamlInterface, &["mli"]),
-    (Pascal, &["pas", "dfm", "dpr", "lpr", "pascal"]),
-    (Perl, &["pm", "pl"]),
-    (Php, &["php"]),
-    (Python, &["py", "py3", "pyi", "bzl"]),
-    (Qml, &["qml"]),
-    (R, &["R", "r", "rd", "rsx"]),
-    (Racket, &["rkt"]),
-    (Ruby, &["rb", "builder", "spec", "rake"]),
-    (Rust, &["rs"]),
-    (Scala, &["scala", "sbt", "sc"]),
-    (Sql, &["sql", "pgsql"]),
-    (Swift, &["swift"]),
-    (Toml, &["toml"]),
-    (TypeScript, &["ts"]),
-    (Tsx, &["tsx"]),
-    (Yaml, &["yaml", "yml"]),
-    (Zig, &["zig"]),
-];
-
-/// Which file names are associated with which languages.
-pub const LANG_FILE_NAMES: &[(Language, &[&str])] = &[
-    (
-        Bash,
-        &[
+/// File globs that identify languages based on the file path.
+pub fn language_globs(language: Language) -> Vec<glob::Pattern> {
+    let glob_strs: &'static [&'static str] = match language {
+        Ada => &["*.ada", "*.adb", "*.ads"],
+        Bash => &[
+            "*.bash",
+            "*.bats",
+            "*.cgi",
+            "*.command",
+            "*.env",
+            "*.fcgi",
+            "*.ksh",
+            "*.sh",
+            "*.sh.in",
+            "*.tmux",
+            "*.tool",
+            "*.zsh",
             ".bash_aliases",
             ".bash_history",
             ".bash_logout",
@@ -265,13 +210,59 @@ pub const LANG_FILE_NAMES: &[(Language, &[&str])] = &[
             "zshenv",
             "zshrc",
         ],
-    ),
-    (CMake, &["CMakeLists.txt"]),
-    (EmacsLisp, &[".emacs", "_emacs", "Cask"]),
-    (Erlang, &["Emakefile"]),
-    (
-        Json,
-        &[
+        C => &["*.c"],
+        Clojure => &[
+            "*.bb", "*.boot", "*.clj", "*.cljc", "*.clje", "*.cljs", "*.cljx", "*.edn", "*.joke",
+            "*.joker",
+        ],
+        CMake => &["*.cmake", "*.cmake.in", "CMakeLists.txt"],
+        CommonLisp => &["*.lisp", "*.lsp", "*.asd"],
+        // Treat .h as C++ rather than C. This is an arbitrary choice, but
+        // C++ is more widely used than C according to
+        // https://madnight.github.io/githut/
+        CPlusPlus => &["*.cc", "*.cpp", "*.h", "*.hh", "*.hpp", "*.ino", "*.cxx"],
+        CSharp => &["*.cs"],
+        Css => &["*.css"],
+        Dart => &["*.dart"],
+        Elm => &["*.elm"],
+        EmacsLisp => &["*.el", ".emacs", "_emacs", "Cask"],
+        Elixir => &["*.ex", "*.exs"],
+        Elvish => &["*.elv"],
+        Erlang => &[
+            "*.erl",
+            "*.app.src",
+            "*.es",
+            "*.escript",
+            "*.hrl",
+            "*.xrl",
+            "*.yrl",
+            "Emakefile",
+        ],
+        Gleam => &["*.gleam"],
+        Go => &["*.go"],
+        Hack => &["*.hack", "*.hck", "*.hhi"],
+        Hare => &["*.ha"],
+        Haskell => &["*.hs"],
+        Hcl => &["*.hcl", "*.nomad", "*.tf", "*.tfvars", "*.workflow"],
+        Html => &["*.html", "*.htm", "*.xhtml"],
+        Janet => &["*.janet", "*.jdn"],
+        Java => &["*.java"],
+        JavaScript => &["*.cjs", "*.js", "*.mjs"],
+        Json => &[
+            "*.json",
+            "*.avsc",
+            "*.geojson",
+            "*.gltf",
+            "*.har",
+            "*.ice",
+            "*.JSON-tmLanguage",
+            "*.jsonl",
+            "*.mcmeta",
+            "*.tfstate",
+            "*.tfstate.backup",
+            "*.topojson",
+            "*.webapp",
+            "*.webmanifest",
             ".arcconfig",
             ".auto-changelog",
             ".c8rc",
@@ -285,10 +276,17 @@ pub const LANG_FILE_NAMES: &[(Language, &[&str])] = &[
             "composer.lock",
             "mcmod.info",
         ],
-    ),
-    (
-        Make,
-        &[
+        JavascriptJsx => &["*.jsx"],
+        Julia => &["*.jl"],
+        Kotlin => &["*.kt", "*.ktm", "*.kts"],
+        Lua => &["*.lua"],
+        Make => &[
+            "*.mak",
+            "*.d",
+            "*.make",
+            "*.makefile",
+            "*.mk",
+            "*.mkfile",
             "BSDmakefile",
             "GNUmakefile",
             "Kbuild",
@@ -303,34 +301,94 @@ pub const LANG_FILE_NAMES: &[(Language, &[&str])] = &[
             "makefile.sco",
             "mkfile",
         ],
-    ),
-    (Python, &["TARGETS", "BUCK", "DEPS"]),
-    (R, &[".Rprofile", "expr-dist"]),
-    (Ruby, &["Gemfile", "Rakefile"]),
-    (
-        Toml,
-        &["Cargo.lock", "Gopkg.lock", "Pipfile", "poetry.lock"],
-    ),
-];
+        Newick => &["*.nhx", "*.nwk", "*.nh"],
+        Nix => &["*.nix"],
+        OCaml => &["*.ml"],
+        OCamlInterface => &["*.mli"],
+        Pascal => &["*.pas", "*.dfm", "*.dpr", "*.lpr", "*.pascal"],
+        Perl => &["*.pm", "*.pl"],
+        Php => &["*.php"],
+        Python => &["*.py", "*.py3", "*.pyi", "*.bzl", "TARGETS", "BUCK", "DEPS"],
+        Qml => &["*.qml"],
+        R => &["*.R", "*.r", "*.rd", "*.rsx", ".Rprofile", "expr-dist"],
+        Racket => &["*.rkt"],
+        Ruby => &[
+            "*.rb",
+            "*.builder",
+            "*.spec",
+            "*.rake",
+            "Gemfile",
+            "Rakefile",
+        ],
+        Rust => &["*.rs"],
+        Scala => &["*.scala", "*.sbt", "*.sc"],
+        Solidity => &["*.sol"],
+        Sql => &["*.sql", "*.pgsql"],
+        Swift => &["*.swift"],
+        Toml => &[
+            "*.toml",
+            "Cargo.lock",
+            "Gopkg.lock",
+            "Pipfile",
+            "poetry.lock",
+        ],
+        TypeScript => &["*.ts"],
+        TypeScriptTsx => &["*.tsx"],
+        Yaml => &["*.yaml", "*.yml"],
+        Zig => &["*.zig"],
+    };
+
+    glob_strs
+        .iter()
+        .map(|name| {
+            glob::Pattern::new(name).expect("Glob in difftastic source should be well-formed")
+        })
+        .collect()
+}
+
+fn looks_like_hacklang(path: &Path, src: &str) -> bool {
+    if let Some(extension) = path.extension() {
+        if extension == "php" && src.starts_with("<?hh") {
+            return true;
+        }
+    }
+
+    false
+}
+
+pub fn guess(
+    path: &Path,
+    src: &str,
+    overrides: &[(glob::Pattern, LanguageOverride)],
+) -> Option<Language> {
+    if let Some(file_name) = path.file_name() {
+        let file_name = file_name.to_string_lossy();
+        for (pattern, lang_override) in overrides {
+            if pattern.matches(&file_name) {
+                match lang_override {
+                    LanguageOverride::Language(lang) => return Some(*lang),
+                    LanguageOverride::PlainText => {
+                        return None;
+                    }
+                }
+            }
+        }
+    }
 
-pub fn guess(path: &Path, src: &str) -> Option<Language> {
     if let Some(lang) = from_emacs_mode_header(src) {
         return Some(lang);
     }
     if let Some(lang) = from_shebang(src) {
         return Some(lang);
     }
-    if let Some(lang) = from_name(path) {
+    if looks_like_hacklang(path, src) {
+        return Some(Language::Hack);
+    }
+    if let Some(lang) = from_glob(path) {
         return Some(lang);
     }
 
-    match path.extension() {
-        Some(extension) => match from_extension(extension) {
-            Some(Language::Php) if src.starts_with("<?hh") => None,
-            language => language,
-        },
-        None => None,
-    }
+    None
 }
 
 /// Try to guess the language based on an Emacs mode comment at the
@@ -375,11 +433,12 @@ fn from_emacs_mode_header(src: &str) -> Option<Language> {
             "perl" => Some(Perl),
             "python" => Some(Python),
             "racket" => Some(Racket),
-            "rjsx" => Some(Jsx),
+            "rjsx" => Some(JavascriptJsx),
             "ruby" => Some(Ruby),
             "rust" => Some(Rust),
             "scala" => Some(Scala),
             "sh" => Some(Bash),
+            "solidity" => Some(Solidity),
             "sql" => Some(Sql),
             "swift" => Some(Swift),
             "toml" => Some(Toml),
@@ -440,14 +499,14 @@ fn from_shebang(src: &str) -> Option<Language> {
     None
 }
 
-fn from_name(path: &Path) -> Option<Language> {
+fn from_glob(path: &Path) -> Option<Language> {
     match path.file_name() {
         Some(name) => {
             let name = name.to_string_lossy().into_owned();
-            for (language, known_file_names) in LANG_FILE_NAMES {
-                for known_file_name in *known_file_names {
-                    if &name == known_file_name {
-                        return Some(*language);
+            for language in Language::iter() {
+                for glob in language_globs(language) {
+                    if glob.matches(&name) {
+                        return Some(language);
                     }
                 }
             }
@@ -458,19 +517,6 @@ fn from_name(path: &Path) -> Option<Language> {
     }
 }
 
-pub fn from_extension(current_extension: &OsStr) -> Option<Language> {
-    let current_extension = current_extension.to_string_lossy();
-
-    for (language, extensions) in LANG_EXTENSIONS {
-        for extension in *extensions {
-            if &*current_extension == *extension {
-                return Some(*language);
-            }
-        }
-    }
-    None
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -479,32 +525,32 @@ mod tests {
     #[test]
     fn test_guess_by_extension() {
         let path = Path::new("foo.el");
-        assert_eq!(guess(path, ""), Some(EmacsLisp));
+        assert_eq!(guess(path, "", &[]), Some(EmacsLisp));
     }
 
     #[test]
     fn test_guess_by_whole_name() {
         let path = Path::new("foo/.bashrc");
-        assert_eq!(guess(path, ""), Some(Bash));
+        assert_eq!(guess(path, "", &[]), Some(Bash));
     }
 
     #[test]
     fn test_guess_by_shebang() {
         let path = Path::new("foo");
-        assert_eq!(guess(path, "#!/bin/bash"), Some(Bash));
+        assert_eq!(guess(path, "#!/bin/bash", &[]), Some(Bash));
     }
 
     #[test]
     fn test_guess_by_env_shebang() {
         let path = Path::new("foo");
-        assert_eq!(guess(path, "#!/usr/bin/env python"), Some(Python));
+        assert_eq!(guess(path, "#!/usr/bin/env python", &[]), Some(Python));
     }
 
     #[test]
     fn test_guess_by_emacs_mode() {
         let path = Path::new("foo");
         assert_eq!(
-            guess(path, "; -*- mode: Lisp; eval: (auto-fill-mode 1); -*-"),
+            guess(path, "; -*- mode: Lisp; eval: (auto-fill-mode 1); -*-", &[]),
             Some(CommonLisp)
         );
     }
@@ -513,7 +559,7 @@ mod tests {
     fn test_guess_by_emacs_mode_second_line() {
         let path = Path::new("foo");
         assert_eq!(
-            guess(path, "#!/bin/bash\n; -*- mode: Lisp; -*-"),
+            guess(path, "#!/bin/bash\n; -*- mode: Lisp; -*-", &[]),
             Some(CommonLisp)
         );
     }
@@ -521,18 +567,34 @@ mod tests {
     #[test]
     fn test_guess_by_emacs_mode_shorthand() {
         let path = Path::new("foo");
-        assert_eq!(guess(path, "(* -*- tuareg -*- *)"), Some(OCaml));
+        assert_eq!(guess(path, "(* -*- tuareg -*- *)", &[]), Some(OCaml));
     }
 
     #[test]
     fn test_guess_by_emacs_mode_shorthand_no_spaces() {
         let path = Path::new("foo");
-        assert_eq!(guess(path, "# -*-python-*-"), Some(Python));
+        assert_eq!(guess(path, "# -*-python-*-", &[]), Some(Python));
     }
 
     #[test]
     fn test_guess_unknown() {
         let path = Path::new("jfkdlsjfkdsljfkdsljf");
-        assert_eq!(guess(path, ""), None);
+        assert_eq!(guess(path, "", &[]), None);
+    }
+
+    #[test]
+    fn test_guess_override() {
+        let path = Path::new("foo.el");
+        assert_eq!(
+            guess(
+                path,
+                "",
+                &[(
+                    glob::Pattern::new("*.el").unwrap(),
+                    LanguageOverride::Language(Css)
+                )]
+            ),
+            Some(Css)
+        );
     }
 }
